@@ -1,5 +1,5 @@
 import math
-from collections import OrderedDict
+from collections import OrderedDict, defaultdict
 from typing import List, Tuple, Union, Dict
 
 import cairo
@@ -7,147 +7,8 @@ from cairo import Context
 
 from hypertrie.colors import *
 from hypertrie.hypertrie2 import HypertrieContext, hash_t, HypertrieNode
-
-
-class PlotStyle:
-    """
-    Defines styles for plot
-    """
-
-    def __init__(self,
-                 padding=8,
-                 hpadding=None,
-                 block_size=25,
-                 corner_radius=None,
-                 dim_tag_size=None,
-                 line_width=2,
-                 font_size=None,
-                 ):
-        self.padding = padding
-
-        if hpadding is None:
-            self.hpadding = self.padding / 2
-        else:
-            self.hpadding = hpadding
-
-        self.block_size = block_size
-
-        if dim_tag_size is None:
-            self.dim_tag_size = self.block_size / 25 * 15
-        else:
-            self.dim_tag_size = dim_tag_size
-
-        if corner_radius is None:
-            self.corner_radius = self.block_size / 5
-        else:
-            self.corner_radius = corner_radius
-
-        self.line_width = line_width
-
-        if font_size is None:
-            self.font_size = self.block_size / 25 * 15
-        else:
-            self.font_size = font_size
-
-        self.dim_font_size = self.dim_tag_size / 25 * 19
-
-        self.sb_padding = self.padding + self.line_width / 2
-        self.db_padding = self.padding + self.line_width
-        self.sb_hpadding = self.hpadding + self.line_width / 2
-        self.db_hpadding = self.hpadding + self.line_width
-
-        self.SUB = str.maketrans("0123456789", "₀₁₂₃₄₅₆₇₈₉")
-        self.SUP = str.maketrans("0123456789", "⁰¹²³⁴⁵⁶⁷⁸⁹")
-
-
-def rounded_rectangle(ctx: Context,
-                      x: float, y: float,
-                      width, height,
-                      fill_color=light_gray_transp,
-                      border_color=None,
-                      aspect=1.0,
-                      corner_radius=None):
-    if corner_radius is None:
-        corner_radius = height / 10
-
-    from math import pi
-    radius = corner_radius / aspect
-    degrees = pi / 180.0
-
-    ctx.new_sub_path()
-    ctx.arc(x + width - radius, y + radius, radius, -90 * degrees, 0 * degrees)
-    ctx.arc(x + width - radius, y + height - radius, radius, 0 * degrees, 90 * degrees)
-    ctx.arc(x + radius, y + height - radius, radius, 90 * degrees, 180 * degrees)
-    ctx.arc(x + radius, y + radius, radius, 180 * degrees, 270 * degrees)
-    ctx.close_path()
-
-    ctx.set_source_rgba(*fill_color)
-    if border_color is None:
-        ctx.fill()
-    else:
-        ctx.fill_preserve()
-        ctx.set_source_rgba(*border_color)
-        ctx.stroke()
-
-
-def arrow(ctx: Context, ps: PlotStyle,
-          start_x, start_y,
-          end_x, end_y,
-          color=black, head_length=4):
-    head_degree = 0.6
-    angle = math.atan2(end_y - start_y, end_x - start_x) + math.pi
-
-    x1 = end_x + head_length * math.cos(angle - head_degree)
-    y1 = end_y + head_length * math.sin(angle - head_degree)
-    x2 = end_x + head_length * math.cos(angle + head_degree)
-    y2 = end_y + head_length * math.sin(angle + head_degree)
-    xm = (x1 + x2) / 2
-    ym = (y1 + y2) / 2
-    # rectangle
-
-    ctx.set_line_width(ps.line_width)
-    ctx.set_source_rgba(*color)
-
-    ctx.new_sub_path()
-    ctx.move_to(end_x, end_y)
-    ctx.line_to(x1, y1)
-    ctx.line_to(x2, y2)
-    ctx.line_to(end_x, end_y)
-    ctx.close_path()
-    ctx.fill()
-
-    ctx.move_to(xm, ym)
-    ctx.line_to(start_x, start_y)
-    ctx.stroke()
-
-
-def rect_with_text(ctx: Context, ps: PlotStyle,
-                   x: float,
-                   y: float,
-                   text,
-                   size: float,
-                   rgba_border=black,
-                   rgba_font=black,
-                   rgba_fill=(0, 0, 0, 0)):
-    # rectangle
-    size = float(size)
-    ctx.set_line_width(ps.line_width)
-    ctx.set_font_size(size / 25 * 19)
-
-    ctx.rectangle(x, y, size, size)
-    if rgba_fill is not None:
-        ctx.set_source_rgba(*rgba_fill)
-        ctx.fill_preserve()
-    ctx.set_source_rgba(*rgba_border)
-    ctx.stroke()
-
-    # text
-    (x_, y_, width, height, dx, dy) = ctx.text_extents(str(text))
-    (ascent, descent, height, max_x_advance, max_y_advance) = ctx.font_extents()
-
-    ctx.move_to(x + (size / 2) - dx / 2, y + (size / 2) + ((ascent - descent) / 2))
-    ctx.set_source_rgba(*rgba_font)
-    ctx.show_text(str(text))
+from hypertrie.hypertrie_commons import HTKey
+from hypertrie.cairo_commons import PlotStyle, rect_with_text, rounded_rectangle, arrow
 
 
 class DimBox:
@@ -258,9 +119,6 @@ def add_referencable_hypertrie_node(ctx: Context, ps: PlotStyle, x: float, y: fl
     return asset.width, asset.height + y_offset
 
 
-class AbsctractNode:
-    pass
-
 
 class UncomressedHypertrieNode:
 
@@ -416,8 +274,8 @@ class CompressedHypertrieNode:
         self.ctx.show_text(size_text)
 
 
-def cairo_draw_hypertrie2_context(ctx: Context, ps: PlotStyle, ht_ctx: HypertrieContext):
-    ctx.select_font_face("Liberation Sans", cairo.FONT_SLANT_NORMAL,
+def cairo_draw_hypertrie2_context(ctx: Context, ps: PlotStyle, ht_ctx: HypertrieContext, level_dxs: Dict[int, float] = None):
+    ctx.select_font_face(ps.font, cairo.FONT_SLANT_NORMAL,
                          cairo.FONT_WEIGHT_NORMAL)
     ctx.set_font_size(ps.font_size)
     ctx.set_line_width(ps.line_width)
@@ -441,120 +299,58 @@ def cairo_draw_hypertrie2_context(ctx: Context, ps: PlotStyle, ht_ctx: Hypertrie
         else:
             return child
 
-    depth = 3
-    nodes: Dict[hash_t, HypertrieNode] = ht_ctx.storage[depth - 1]
     x = 0
     y = ps.line_width / 2
     x_max = x
     x_offset = 0
     y_offset = 0
-    for node_hash, node in nodes.items():
-        id = next_id
-        inc_id()
+    if level_dxs is None:
+        level_dxs: Dict[int, float] = defaultdict(float)
+    for depth in range(ht_ctx.max_depth, 0, -1):
 
-        data = [list() for _ in range(depth)]
-        for dim in range(depth):
-            entries = sorted(node.children[dim].items())
+        x_offset = level_dxs[depth]
+        y += y_offset
+        y_offset = 0
 
-            data[dim] = [(key_part, map_id(child)) for key_part, child in entries]
+        nodes: Dict[hash_t, HypertrieNode] = ht_ctx.storage[depth - 1]
+        hash2id = next_hash2id.copy()
+        next_hash2id.clear()
+        if depth != ht_ctx.max_depth:
+            assert len(nodes) == len(hash2id)
 
-        tmp_x_offset, y_offset = add_referencable_hypertrie_node(ctx, ps,
-                                                                 x + x_offset, y,
-                                                                 id=id,
-                                                                 size=node.size,
-                                                                 ref_count=node.count,
-                                                                 data=data,
-                                                                 node=node)
-        x_offset += tmp_x_offset + 15
-    y_offset += 15
-    x_max = max(x_max, x + x_offset)
-    ####
-    depth = 2
-    x_offset = 0
-    y += y_offset
-    y_offset = 0
-    hash2id = next_hash2id.copy()
-    next_hash2id.clear()
-    nodes: Dict[hash_t, HypertrieNode] = ht_ctx.storage[depth - 1]
-    assert len(nodes) == len(hash2id)
-    for node_hash, id in hash2id.items():
-        node = nodes[node_hash]
+        for node_hash, node in nodes.items():
+            if node_hash not in hash2id:
+                hash2id[node_hash] = next_id
+                id = next_id
+                inc_id()
+            else:
+                id = hash2id[node_hash]
 
-        data = [list() for _ in range(depth)]
-        if node.size == 1:
-            data = list(node.children)
-        else:
-            for dim in range(depth):
-                entries = sorted(node.children[dim].items())
+            if node.size == 1:
+                assert depth != 1
+                data = list(node.children)
+            else:
+                data = [list() for _ in range(depth)]
 
-                data[dim] = [(key_part, map_id(child)) for key_part, child in entries]
+                for dim in range(depth):
+                    if depth > 1:
+                        entries = sorted(node.children[dim].items())
+                        data[dim] = [(key_part, map_id(child)) for key_part, child in entries]
+                    else:
+                        entries = sorted(node.children[dim])
+                        data[dim] = [(key_part, "") for key_part in entries]
 
-        tmp_x_offset, tmp_y_offset = add_referencable_hypertrie_node(
-            ctx, ps, x + x_offset, y,
-            id=id,
-            size=node.size,
-            ref_count=node.count,
-            data=data,
-            node=node)
-        x_offset += tmp_x_offset + 15
-        y_offset = max(tmp_y_offset, y_offset)
-    y_offset += 15
-    x_max = max(x_max, x + x_offset)
-    ####
-    depth = 1
-    y += y_offset
-    y_offset = 0
-    x_offset = 0
-    hash2id = next_hash2id.copy()
-    next_hash2id.clear()
-    nodes: Dict[hash_t, HypertrieNode] = ht_ctx.storage[depth - 1]
-    assert len(nodes) == len(hash2id)
-    for node_hash, id in hash2id.items():
-        node = nodes[node_hash]
-
-        data = [list() for _ in range(depth)]
-        assert node.size != 1
-
-        for dim in range(depth):
-            entries = sorted(node.children[dim])
-
-            data[dim] = [(key_part, "") for key_part in entries]
-
-        tmp_x_offset, tmp_y_offset = add_referencable_hypertrie_node(
-            ctx, ps, x + x_offset, y,
-            id=id,
-            size=node.size,
-            ref_count=node.count,
-            data=data,
-            node=node)
-        x_offset += tmp_x_offset + 15
-        y_offset = max(tmp_y_offset, y_offset)
-    x_max = max(x_max, x + x_offset)
-    x_max -= 15
-    return x_max, y + y_offset
-
-#
-# ht_ctx = HypertrieContext()
-# ht_ctx.create_hypertrie(
-#     keys=[
-#         (1, 2, 3),
-#         (1, 2, 4),
-#         (5, 2, 4),
-#         (5, 2, 3),
-#     ]
-# )
-# ht_ctx.create_hypertrie(
-#     keys=[
-#         (1, 2, 3),
-#         (1, 2, 4),
-#         (3, 2, 4),
-#         (3, 2, 5),
-#         (4, 2, 3),
-#         (4, 2, 5),
-#         (3, 6, 7),
-#         (5, 6, 7),
-#
-#     ]
-# )
-#
-# draw_hypertrie2_context(ht_ctx)
+            tmp_x_offset, y_offset = add_referencable_hypertrie_node(
+                ctx, ps,
+                x + x_offset, y,
+                id=id,
+                size=node.size,
+                ref_count=node.count,
+                data=data,
+                node=node)
+            x_offset += tmp_x_offset + 15
+        y_offset += 15
+        level_dxs[depth] = x + x_offset - 15
+        x_max = max(x_max, level_dxs[depth])
+    y_offset -= 15
+    return x_max, y + y_offset, level_dxs
\ No newline at end of file
